import asyncio
import re
from .const import *
from .accountingnode import get_parent_orgid
from .excep import *
from .getaccount import get_account, getAccountByName
from appPublic.uniqueID import getID
from appPublic.log import debug, exception
from sqlor.dbpools import DBPools
from appPublic.timeUtils import curDateString
# from .argsconvert import ArgsConvert
from appPublic.argsConvert import ArgsConvert
from datetime import datetime

accounting_config = None

async def get_accounting_config(sor):
	global accounting_config
	if accounting_config:
		return accounting_config
	recs = await sor.R('accounting_config', {})
	if len(recs) > 0:
		accounting_config = recs
		return accounting_config
	return None

class Accounting:
	"""
	需要caller功能：
	caller中要有分录中的变量
	get_accounting_orgid(leg) 获得记账机构
	get_account(leg，accounting_orgid)	获得记账账号（通过科目，机构类型，账务机构确定一个唯一的账号）
	"""
	def __init__(self, caller):
		self.caller = caller
		self.curdate = caller.curdate
		self.realtimesettled = False
		self.curdte = caller.curdate
		self.timestamp = caller.timestamp
		self.billid = caller.billid
		self.action = caller.action
		self.summary = f'{self.caller.orderid}:{self.caller.billid}'
		self.providerid = caller.providerid
		self.productid = caller.productid
		self.resellerid = caller.resellerid
		self.customerid = caller.customerid
		self.own_salemode = None
		self.reseller_salemode = None
		self.variable = caller.variable

	async def setup_accounting_legs(self):
		global accounting_config
		action = self.action.split('_')[0]
		await get_accounting_config(self.sor)
		self.accounting_legs = [r.copy() for r in accounting_config 
									if r.action == action ]
		debug(f'{self.accounting_legs=}')
		rev = self.action.endswith('_REVERSE')
		for l in self.accounting_legs:
			if rev:
				l['acc_dir'] = DEBT if l['accounting_dir'] == CREDIT else CREDIT
			else:
				l['acc_dir'] = l['accounting_dir']
			ac = ArgsConvert('${', '}$')
			try:
				amtstr = ac.convert(l['amt_pattern'],
							self.variable.copy()
				)
				debug(f'{l["amt_pattern"]=}, {amtstr=}, {self.variable=}')
				if isinstance(amtstr, str):
					l['amount'] = eval(amtstr)
				else:
					l['amount'] = amtstr
 
			except Exception as e:
				exception(f'{e=}, {l['amt_pattern']}, {self.variable=}')
				raise e

			if l['amount'] is None:
				debug(f'amount is None:{l["amt_pattern"]}, {self.variable=},{self.caller.bill=}')
				raise AccountingAmountIsNone(self.caller.billid)
			
	def check_accounting_balance(self, legs):
		debt_balance = 0.0
		credit_balance = 0.0
		for l in legs:
			if l['acc_dir'] == DEBT:
				debt_balance += l['amount']
			else:
				credit_balance += l['amount']
		if abs(credit_balance - debt_balance) >= 0.01:
			e = Exception('accounting legs not balance')
			exception(f'{legs=}, {e=}')
			raise e

	async def do_accounting(self, sor):
		self.sor = sor
	
		await self.setup_accounting_legs()
		debug(f'do_accounting() ...{self.accounting_legs=}')
		self.check_accounting_balance(self.accounting_legs)
		for leg in self.accounting_legs:
			accounting_orgid = await self.caller.get_orgid_by_trans_role(sor, leg.accounting_orgtype)
			orgid = await self.caller.get_orgid_by_trans_role(sor, leg.orgtype)
			org1id = None if leg.org1type is None else \
						await self.caller.get_orgid_by_trans_role(sor, leg.org1type)
			acc = await get_account(sor, accounting_orgid, orgid, leg.subjectid, org1id=org1id)
			if acc is None:
				debug(f'can not get accountid {accounting_orgid=}, {orgid=},{leg.subjectid=}, {org1id=}')
				raise AccountIdNone(accounting_orgid, orgid, leg['subjectid'])
			leg['_accounting_orgid'] = accounting_orgid
			leg['_orgid'] = orgid
			leg['_org1id'] = org1id
			leg['acc'] = acc
			await self.leg_accounting(sor, acc.id, leg)

	async def write_settle_log(self):
		sale_mode = {
			SALEMODE_DISCOUNT:'0',
			SALEMODE_REBATE:'1',
			SALEMODE_FLOORPRICE:'2'
		}
		ns = {
			'id':getID(),
			'accounting_orgid':self.accounting_orgid,
			'providerid':self.providerid,
			'sale_mode':sale_mode.get(self.own_salemode),
			'settle_date':self.curdate,
			'settle_amt':self.accounting_legs[-1]['amount']
		}
		
		sor = self.sor
		await sor.C('settle_log', ns)

	async def overdraw_check(self, sor, accid, leg, tryAgain=True):
		if accid is None:
			raise AccountIdNone()

		sql0 = "select max(acc_date) as acc_date from acc_balance where accountid=${accid}$"
		recs = await sor.sqlExe(sql0, {'accid':accid})
		acc_date = recs[0]['acc_date']
		bal = {}
		if acc_date is not None:
			if acc_date > self.curdate:
				raise FutureAccountingExist(accid, self.curdate, acc_date)
			ns={'accid':accid, 'acc_date':acc_date}
			r = await sor.sqlExe("""select * from acc_balance 
where accountid=${accid}$ 
	and acc_date = ${acc_date}$""", ns.copy())
			if len(r) > 0:
				bal = r[0]

		accs = await sor.R('account', {'id':accid})
		if len(accs) == 0:
			raise AccountNoFound(accid)

		acc = accs[0]
		acc['acc_date'] = self.curdate
		acc['balance'] = bal.get('balance', 0)

		if acc.get('balance') is None:
			acc['balance'] = 0
		if acc['balance_at'] == DEBT and leg['acc_dir'] == CREDIT \
				or  acc['balance_at'] == CREDIT and leg['acc_dir'] == DEBT:
			if int(acc['balance']*10000) - int(leg['amount']*10000) < 0:
				if tryAgain:
					await asyncio.sleep(1.5);
					return await self.overdraw_check(sor, accid, leg, tryAgain=False)
				else:
					print(f"{acc['balance_at']=}, {leg=}")
					raise AccountOverDraw(accid, acc['balance'], leg['amount'])
			leg['new_balance'] = acc['balance'] - leg['amount']
		else:
			leg['new_balance'] = acc['balance'] + leg['amount']
		
	async def leg_accounting(self, sor, accid, leg):
		# print(f'leg_accounting(), {accid=}, {leg=}')
		if leg['amount'] < 0.0001:
			return
		await self.overdraw_check(sor, accid, leg)
		# write acc_balance
		sql = """select * from acc_balance 
where accountid=${accid}$ 
	and acc_date = ${curdate}$"""
		recs = await sor.sqlExe(sql, {'accid':accid, 'curdate':self.curdate})
		if len(recs) == 0:
			ns = {
				'id':getID(),
				'accountid':accid,
				'acc_date':self.curdate,
				'balance':leg['new_balance']
			}
			await sor.C('acc_balance', ns.copy())
		else:
			ns = recs[0]
			ns['balance'] = leg['new_balance']
			await sor.U('acc_balance', ns.copy())

		# summary = self.summary
		ns = {
			'id':getID(),
			'accounting_orgid' : leg['accounting_orgid'],
			'billid' : self.billid,
			'description' : self.summary,
			'participantid' : leg['orgid'],
			'participanttype' : leg['orgtype'],
			'subjectname' : leg['subjectname'],
			'accounting_dir': leg['accounting_dir'],
			'amount' : leg['amount']
		}
		await sor.C('bill_detail', ns)
		logid = getID()
		ns = {
			'id':logid,
			'accountid':accid,
			'acc_date':self.curdte,
			'acc_timestamp':self.timestamp,
			'acc_dir':leg['acc_dir'],
			'summary':self.summary,
			'amount':leg['amount'],
			'billid':self.billid
		}
		await sor.C('accounting_log', ns.copy())
		ns = {
			'id':getID(),
			'accountid':accid,
			'acc_date':self.curdate,
			'acc_timestamp':self.timestamp,
			'acc_dir':leg['acc_dir'],
			'summary':self.summary,
			'amount':leg['amount'],
			'balance':leg['new_balance'],
			'acclogid':logid
		}
		await sor.C('acc_detail', ns.copy())

